/*
 * AlsaMicGrabber.cpp
 *
 *  Created on: Apr 18, 2012
 *      Author: mschoepf
 */

#include "AlsaMicGrabber.h"

using namespace std;


void
AlsaMicGrabber::init ()
{
	int err;
	unsigned int rate = sampleRate;
	snd_pcm_hw_params_alloca(&hwParams);
	if ((err = snd_pcm_open(&pcmHandle, hardwareDeviceName.c_str(), SND_PCM_STREAM_CAPTURE, 0)) < 0)
	{
		cerr << "Could not open sound device '" << hardwareDeviceName << "', reason: " << snd_strerror(err) << endl;
		throw AlsaPcmDeviceNotFoundException();
	}

	if ((err = snd_pcm_hw_params_any(pcmHandle,hwParams)) < 0)
	{
		cerr << "Could not get sound hardware params, reason: " << snd_strerror(err) << endl;
		throw AlsaPcmConfigureException();
	}

	if ((err = snd_pcm_hw_params_set_access(pcmHandle,hwParams,SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
	{
		cerr << "Could not set sound buffer access, reason: " << snd_strerror(err) << endl;
		throw AlsaPcmConfigureException();
	}

	if ((err = snd_pcm_hw_params_set_format(pcmHandle,hwParams,SND_PCM_FORMAT_S16_LE)) < 0)
	{
		cerr << "Could not set sound format, reason: " << snd_strerror(err) << endl;
		throw AlsaPcmConfigureException();
	}
	if ((err = snd_pcm_hw_params_set_rate_near(pcmHandle, hwParams, &rate, 0)) < 0)
	{
		cerr << "Could not set sound sample rate, reason: " << snd_strerror(err) << endl;
		throw AlsaPcmConfigureException();
	}
	if ((err = snd_pcm_hw_params_set_channels(pcmHandle,hwParams,1)) < 0)
	{
		cerr << "Could not set channels, reason: " << snd_strerror(err) << endl;
		throw AlsaPcmConfigureException();
	}
	if (rate != sampleRate)
	{
		cout << "Warning: AlsaMicGrabber wanted " << sampleRate << " Hz sample rate, got " << rate << " Hz sample rate";
	}

	if ((err = snd_pcm_hw_params (pcmHandle,hwParams)) < 0)
	{
		cerr << "Could not set sound hardware parameters, reason: " << snd_strerror (err) << endl;
		throw AlsaPcmConfigureException();
	}

	readThreadPtr.reset(new boost::thread(&AlsaMicGrabber::grab,this));
}


void
AlsaMicGrabber::grab()
{
        bool keepRunning=true;
	int pcmreturn;
        size_t frames_total_grabbed=0;
	short* localbuffer;
        short** emitbuffer;        
        localbuffer = new short[sampleSize];
        unsigned int pushBufferSize = 2;
        emitbuffer = new short*[pushBufferSize];        
        for (int i=0; i < pushBufferSize; i++)
           emitbuffer[i] = new short[sampleSize];           
	if ((pcmreturn = snd_pcm_prepare (pcmHandle)) < 0)
	{
		cerr << "Error during sound hardware prepare: " << snd_strerror(pcmreturn) << endl;
		throw AlsaPcmConfigureException();
	}
	while(keepRunning)
	{
		try
		{
			pcmreturn = snd_pcm_readi(pcmHandle,localbuffer,sampleSize);                        
			if (pcmreturn < 0)
			{
				printf("Stream status error: %s\n", snd_strerror(pcmreturn));
				snd_pcm_prepare(pcmHandle);
				cerr << "<<<<<<<<<<<<<<< Buffer Overrun >>>>>>>>>>>>>>>\n";
			}
                        {
                                boost::unique_lock<boost::mutex> lock (bufferMutex);
                                memcpy(buffer,localbuffer,bufferSize);
                                memcpy(emitbuffer[frames_total_grabbed%pushBufferSize],localbuffer,bufferSize);
                        }
                        emit pushGrab(emitbuffer[frames_total_grabbed%pushBufferSize],sampleSize);
                        frames_total_grabbed++;
			calc_dB();
			//std::cerr << "emitting signal " << std::endl;
			emit updated_dB_and_clipped (static_cast<float> (signalNoiseRatioChunk), clippedChunk);
			//cerr << "Read from device\n";			                        
                        boost::this_thread::interruption_point();
		}
		catch (boost::thread_interrupted &ti)
		{
			keepRunning = false;
		}
	}
        delete localbuffer;
}

void
AlsaMicGrabber::getLastBuffer (std::vector<short> &buffer)
{
    boost::unique_lock<boost::mutex> lock (bufferMutex);
    memcpy (buffer.data(),this->buffer,bufferSize);
}

void
AlsaMicGrabber::getLastBufferBlocking (std::vector<short> &buffer)
{
    while (true)
    {
        boost::unique_lock<boost::mutex> lock (bufferMutex);
        if (newDataAvailable)
        {
            memcpy (buffer.data(),this->buffer,bufferSize);
            newDataAvailable = false;
        }
    }
 }

AlsaMicGrabber::AlsaMicGrabber()
{
	hardwareDeviceName = "plughw:0,0";
	signalNoiseRatioTotal = 0;
	clippedTotal = 0;
	totalChunksProcessed = 0;
	init();
}

AlsaMicGrabber::AlsaMicGrabber(std::string DeviceName)
{
	hardwareDeviceName = DeviceName;
	init();
}

AlsaMicGrabber::~AlsaMicGrabber() {
	readThreadPtr->interrupt();
	readThreadPtr->join();
        snd_pcm_close(pcmHandle);
}

void
AlsaMicGrabber::calc_dB()
{
	unsigned int e_sum = 0;
	float e_avg, ratio;
	float db;
	unsigned int clipped = 0;
	float e_ref = 4.5; //This is the default from the ISR sources

	{
		boost::unique_lock<boost::mutex> bufferMutex;
		for (int i = 0; i < sampleSize; ++i)
		{
		
			if ((buffer[i] == SHRT_MIN) || (buffer[i] == SHRT_MAX))
			{
				clipped++;
			}
			e_sum += abs(buffer[i]);
		}
	}
        //std::cerr << "amg e_sum = " << e_sum << std::endl;
	e_avg = static_cast<float> (e_sum) / static_cast<float> (sampleSize);

	ratio = e_avg / e_ref;
	db = (ratio > 1) ? 20.0 * log10(ratio) : 0;

	signalNoiseRatioChunk = db;
	signalNoiseRatioTotal += db;
	++totalChunksProcessed;
	clippedTotal += clipped;
	clippedChunk = clipped;

}
