/*
 * AlsaMicGrabber.h
 *
 *  Created on: Apr 18, 2012
 *      Author: mschoepf
 */

#ifndef ALSAMICGRABBER_H_
#define ALSAMICGRABBER_H_

#include <alsa/asoundlib.h>
#include <string>
#include <iostream>
#include <vector>
#include <boost/thread.hpp>
#include <Qt/qobject.h>
#include "defaults.h"

typedef struct alsa_sound_chunk_t {
    short soundchunk[ALSAMICGRABBER_SAMPLESIZE];
} AlsaSoundChunkT;

class AlsaPcmDeviceNotFoundException: public std::exception {
public:
	virtual const char* what() const throw()
	  {
	    return "Alsa device not found!";
	  }
};

class AlsaPcmConfigureException: public std::exception {
public:
	virtual const char* what() const throw()
	  {
	    return "Alsa device cannot be configured!";
	  }
};

class AlsaMicGrabber : public QObject {

	Q_OBJECT

private:
	snd_pcm_t *pcmHandle;
	snd_pcm_hw_params_t *hwParams;
	std::string hardwareDeviceName;
	static const unsigned int sampleRate = ALSAMICGRABBER_SAMPLERATE;
	static const unsigned int sampleSize = ALSAMICGRABBER_SAMPLESIZE;
	static const unsigned int bufferSize = ALSAMICGRABBER_BUFFERSIZE;
	boost::shared_ptr<boost::thread> readThreadPtr;
	boost::mutex bufferMutex;
        bool newDataAvailable;
	short buffer[sampleSize];
	unsigned int clippedTotal;
	unsigned int clippedChunk;
	float signalNoiseRatioTotal;
	float signalNoiseRatioChunk;
	unsigned long int totalChunksProcessed;
	void init();
	void grab();
	void calc_dB();

signals:
	void updated_dB_and_clipped (float db, bool clipped);
        void pushGrab (short* chunk, unsigned int size);

public slots:

public:
	void getLastBuffer (std::vector<short> &buffer);
        void getLastBufferBlocking (std::vector<short> &buffer);
	AlsaMicGrabber();
	AlsaMicGrabber(std::string DeviceName);
	virtual ~AlsaMicGrabber();
};

#endif /* ALSAMICGRABBER_H_ */
