/* 
 * File:   jsgfParser.cpp
 * Author: mschoepf
 * 
 * Created on June 2, 2013, 2:35 PM
 */

#include "JsgfParser.h"
#include <cstdlib>
#include <iostream>


using namespace std;



namespace jsgfParserTypes {


    void tab(int indent) {
        for (int i = 0; i < indent; ++i)
            std::cout << ' ';
    }

    void TermWrapperTPrinter::operator()(TermT const& t) const {
        TermTPrinter(indent + tabsize)(t);
    }

    void TermWrapperTPrinter::operator()(TermLeafT const& tl) const {
        tab(indent + tabsize);
        std::cout << "TermLeaf size = " << tl.symbols.size() << std::endl;
        for (std::vector<jsgfParserTypes::SymbolT>::size_type i = 0; i < tl.symbols.size(); i++) {
            tab(indent + tabsize);
            std::cout << " '" << tl.symbols.at(i).name << "' type = " << tl.symbols.at(i).type << std::endl;
        }
    }

    void TermTPrinter::operator()(TermT const& t) const {
        tab(indent);
        if (t.optional)
            std::cout << "[" << std::endl;
        else
            std::cout << '{' << std::endl;
        tab(indent);
        std::cout << "#Children = " << t.children.size() << std::endl;

        BOOST_FOREACH(TermWrapperT const& node, t.children) {
            if (t.orTerm) {
                tab(indent);
                std::cout << "|" << std::endl;
            }
            jsgfParserTypes::TermWrapperTPrinter myTermWrapperTPrinter(indent);
            boost::apply_visitor(myTermWrapperTPrinter, node);
        }
        tab(indent);
        if (t.optional)
            std::cout << "]" << std::endl;
        else
            std::cout << '}' << std::endl;
    }

    void TermWrapperExtractor::operator ()(TermT const& t) const {
        TermTExtractor myExtractor(myNames);
        myExtractor(t);
    }

    void TermWrapperExtractor::operator()(TermLeafT const& tl) const {
        for (std::vector<jsgfParserTypes::SymbolT>::size_type i = 0; i < tl.symbols.size(); i++) {
            if (tl.symbols.at(i).type == 1) {
                std::string s(tl.symbols.at(i).name.begin(), tl.symbols.at(i).name.end());
                myNames.push_back(tl.symbols.at(i).name);
            }
        }
    }

    void TermTExtractor::operator()(TermT const& t) const {

        BOOST_FOREACH(TermWrapperT const& node, t.children) {
            boost::apply_visitor(TermWrapperExtractor(myNames), node);
        }
    }

    std::string trim(const std::string& str,
            const std::string& whitespace) {
        std::string::size_type strBegin = str.find_first_not_of(whitespace);
        if (strBegin == std::string::npos)
            return ""; // no content

        std::string::size_type strEnd = str.find_last_not_of(whitespace);
        std::string::size_type strRange = strEnd - strBegin + 1;

        return str.substr(strBegin, strRange);
    }

    std::vector<TermWrapperT> giveChildrenAndDelete::operator()(TermT &t) const {
        std::vector<TermWrapperT> temp;
        for (std::vector<TermWrapperT>::iterator it = t.children.begin(); it != t.children.end(); it++) {
            temp.push_back(*it);
        }
        t.children.clear();
        return temp;
    }

    std::vector<TermWrapperT> giveChildrenAndDelete::operator()(TermLeafT & tl) const {
        std::vector<TermWrapperT> temp;
        return temp;
    }


    void TermWrapperSanitizer::operator() (TermT &t) const {

        if (t.orTerm) {
            isOrTerm = true;

        }
        TermSanitizer()(t);
    }

    void TermWrapperSanitizer::operator() (TermLeafT & tl) const {
        for (std::vector<jsgfParserTypes::SymbolT>::size_type i = 0; i < tl.symbols.size(); i++) {
            std::string s = trim(tl.symbols.at(i).name);
            tl.symbols.at(i).name = s;
        }
    }

    void TermSanitizer::operator()(TermT & t) const {

        BOOST_FOREACH(TermWrapperT & node, t.children) {
            bool isOrTerm = false;
            boost::apply_visitor(TermWrapperSanitizer(isOrTerm), node);
            if (isOrTerm) {
                if (!t.orTerm) {
                    t.orTerm = true;
                }
                //std::cout << "Found orTerm" << std::endl;
                std::vector<TermWrapperT> temp = boost::apply_visitor(giveChildrenAndDelete(), node);
                t.children.pop_back();
                for (unsigned int i = 0; i < temp.size(); i++) {
                    t.children.push_back(temp.at(i));
                }
            }
        }
    }

    void TermWrapperRuleGenerator::operator ()(TermT & t) const {
        TermTRuleGenerator myGenerator(myRules, myNamesToRules, myNonterminal);
        myGenerator(t);

    }

    void TermWrapperRuleGenerator::operator()(TermLeafT & tl) const {
        if (0 == tl.symbols.size())
            return; //Save a lot of time and some if cases below
        std::vector<SymbolT>::size_type myRulesLocalStart = myRules.size();
        for (std::vector<jsgfParserTypes::SymbolT>::size_type i = 0; i < tl.symbols.size(); i++) {
            if (0 == tl.symbols.at(i).type) {
                myRules.push_back(new ruleT(qi::string(trim(tl.symbols.at(i).name))[px::push_back(qi::_val, px::val(trim(tl.symbols.at(i).name))) /*qi::_val = px::val('<') + px::val(myNonterminal) + px::val('>') + px::val(trim(tl.symbols.at(i).name)) + px::val("</") + px::val(myNonterminal) + px::val('>')*/]));
                (*myRules.back()).name(std::string("T:").append(trim(tl.symbols.at(i).name)));
            } else {
                nameToRuleT::iterator pos = myNamesToRules.find(trim(tl.symbols.at(i).name));
                if (pos != myNamesToRules.end()) {
                    if (optional) {
                        myRules.push_back(new ruleT((*myRules.at(pos->second))[qi::_val = px::bind(&nonTerminalInserter, qi::_1, px::val(trim(tl.symbols.at(i).name)))] || qi::eps));
                        (*myRules.back()).name(std::string("+N:").append(trim(tl.symbols.at(i).name)));
                    } else {
                        myRules.push_back(new ruleT((*myRules.at(pos->second))[qi::_val = px::bind(&nonTerminalInserter, qi::_1, px::val(trim(tl.symbols.at(i).name)))]));
                        (*myRules.back()).name(std::string("N:").append(trim(tl.symbols.at(i).name)));
                    }
                } else {
                    std::cout << "Rule mismatch!! Abort!" << std::endl;
                    abort();
                }
            }
        }
        ruleTPtr r = new ruleT((*myRules.at(myRulesLocalStart)));
        r->name(myRules.at(myRulesLocalStart)->name());
        for (std::vector<SymbolT>::size_type i = (myRulesLocalStart + 1); i < myRules.size(); i++) {
            (*r) = (r->copy() >> ((*myRules.at(i))))[qi::_val = px::bind(&concat2stdvectors, qi::_1, qi::_2)];
            r->name(r->name() + " > " + myRules.at(i)->name());
        }
        myRules.push_back(r);
        //std::cout << "TermLeaf created rule " << myRules.back()->name() << std::endl;
    }

    void TermTRuleGenerator::operator()(TermT & t) const {
        std::vector<std::vector<ruleTPtr>::size_type> indices;

        BOOST_FOREACH(TermWrapperT & node, t.children) {
            std::vector<ruleTPtr>::size_type rulesSize = myRules.size();
            boost::apply_visitor(TermWrapperRuleGenerator(myRules, myNamesToRules, myNonterminal, t.optional), node);
            if (rulesSize != myRules.size()) {
                indices.push_back(myRules.size() - 1);
            }
        }
        ruleTPtr r = NULL;
        if (t.orTerm) {
            r = NULL;
            if (indices.size() > 1) {
                r = new ruleT((*myRules.at(indices.at(0))));
                if (t.optional)
                    r->name("[" + (*myRules.at(indices.at(0))).name());
                else
                    r->name("(" + (*myRules.at(indices.at(0))).name());
                for (std::vector<ruleTPtr>::size_type i = 1; i < indices.size(); i++) {
                    (*r) = ((*r).copy() | (*myRules.at(indices.at(i))));
                    r->name(r->name() + " | " + (*myRules.at(indices.at(i))).name());
                }
                if (t.optional) {
                    r->name(r->name() + "]");
                    (*r) = -(*r).copy();
                } else
                    r->name(r->name() + ")");
                myRules.push_back(r);
                //std::cout << "Composed rule " << r->name() << std::endl;
            }
        } else {
            if (indices.size() > 0) {
                if (t.optional)
                    r = new ruleT((*myRules.at(indices.at(0))) | qi::eps);
                else
                    r = new ruleT((*myRules.at(indices.at(0))));
                if (t.optional)
                    r->name("[" + (*myRules.at(indices.at(0))).name());
                else
                    r->name("(" + (*myRules.at(indices.at(0))).name());

                for (std::vector<ruleTPtr>::size_type i = 1; i < indices.size(); i++) {
                    if (t.optional)
                        (*r) = -((*r).copy() >> (*myRules.at(indices.at(i))))[qi::_val = px::bind(&concat2stdvectors, qi::_1, qi::_2)];
                    else
                        (*r) = ((*r).copy() >> (*myRules.at(indices.at(i))))[qi::_val = px::bind(&concat2stdvectors, qi::_1, qi::_2)];
                    r->name(r->name() + " >> " + (*myRules.at(indices.at(i))).name());
                }
                if (t.optional) {
                    r->name(r->name() + "]");
                    (*r) = -((*r).copy() || qi::eps);
                } else
                    r->name(r->name() + ")");

                myRules.push_back(r);
                //std::cout << "Composed rule " << r->name() << std::endl;
            }
        }
    }

    void
    printNonterminal(const std::string s1, const std::string s2) {
        std::cout << " s1='" << s1 << "' s2 = '" << s2 << "'" << std::endl;
    }

    std::vector<std::string>
    concat2stdvectors(const std::vector<std::string> &v1, const std::vector<std::string> &v2) {
        std::vector<std::string> result(v1);
        std::cout << "Vector1: ";
        for (std::vector<std::string>::const_iterator it = v1.begin(); it != v1.end(); it++)
            std::cout << "[" << *it << "] ";
        std::cout << std::endl;
        std::cout << "Vector2: ";
        for (std::vector<std::string>::const_iterator it = v2.begin(); it != v2.end(); it++)
            std::cout << "[" << *it << "] ";
        std::cout << std::endl;
        result.insert(result.end(), v2.begin(), v2.end());
        return result;
    }

    std::vector<std::string>
    nonTerminalInserter(std::vector<std::string> &v, const std::string &s1) {
        for (std::vector<std::string>::iterator it = v.begin(); it != v.end(); it++) {
            std::string buffer = *it;
            (*it) = std::string("<nonterminal name=\"" + s1 + "\">").append(buffer).append("</nonterminal>");
        }
        return v;
    }

    void
    printStringVector(std::vector<std::string> &v) {
        std::cout << "Printing vector: ";
        for (std::vector<std::string>::iterator it = v.begin(); it != v.end(); it++) {
            std::cout << "[" << *it << "] ";
        }
        std::cout << std::endl;
    }

    jsgfParserTypes::TermT
    makePTerm(jsgfParserTypes::TermT &t) {
        t.optional = false;
        t.orTerm = false;
        return t;
    }

    jsgfParserTypes::TermT
    makeOptTerm(jsgfParserTypes::TermT &t) {
        t.optional = true;
        t.orTerm = false;
        return t;
    }

    jsgfParserTypes::TermT
    makeOrTerm(jsgfParserTypes::TermT &t) {
        t.optional = false;
        t.orTerm = true;
        return t;
    }
}