/* 
 * File:   pocketSphinxWidget.cpp
 * Author: mschoepf
 * 
 * Created on May 14, 2013, 5:04 PM
 */

#include <qt4/Qt/qmessagebox.h>
#include <boost/smart_ptr/make_shared.hpp>
#include <boost/filesystem.hpp>


#include "PocketSphinxWidget.h"
#include "PocketsphinxAdapter.h"
#include "publishRsb.h"

boost::shared_ptr<rsbRemoteReceiver> pocketSphinxWidget::myRemoteReceiver;

void
pocketSphinxWidget::setupUI(std::string name, std::string hmm, std::string lm, std::string dict, std::string jsgf, std::string rsb, bool bestpath){
    formLayout = boost::make_shared<QGridLayout>();
    nameLabel = boost::make_shared<QLabel>("Name: ");
    nameLineEdit = boost::make_shared<QLineEdit>(name.c_str());
    hmmLabel = boost::make_shared<QLabel>("HMM Directory: ");
    hmmLineEdit = boost::make_shared<QLineEdit> (hmm.c_str());
    hmmBrowseButton = boost::make_shared<QPushButton> ("Browse...");
    lmLabel = boost::make_shared<QLabel> ("Language Model: ");
    lmLineEdit = boost::make_shared<QLineEdit> (lm.c_str());
    lmBrowseButton = boost::make_shared<QPushButton> ("Browse...");
    dictLabel = boost::make_shared<QLabel>("Dictionary: ");
    dictLineEdit = boost::make_shared<QLineEdit> (dict.c_str());
    dictBrowseButton = boost::make_shared<QPushButton> ("Browse...");
    jsgfLabel = boost::make_shared<QLabel> ("JSGF: ");
    jsgfLineEdit = boost::make_shared<QLineEdit> (jsgf.c_str());
    jsgfBrowseButton = boost::make_shared<QPushButton>("Browse...");
    rsbLabel = boost::make_shared<QLabel> ("RSB Scope: ");
    rsbLineEdit = boost::make_shared<QLineEdit>(rsb.c_str());
    loadButton = boost::make_shared<QPushButton>("Load");
    bestPathCeckBox = boost::make_shared<QCheckBox>("bestpath");    
    bestPathCeckBox->setChecked(bestpath);
    errorMessageDialog = boost::make_shared<QErrorMessage> (this);
    formLayout->addWidget(nameLabel.get(),0,0);
    formLayout->addWidget(nameLineEdit.get(),0,1);
    formLayout->addWidget(hmmLabel.get(),1,0);
    formLayout->addWidget(hmmLineEdit.get(),1,1);
    formLayout->addWidget(hmmBrowseButton.get(),1,2);
    formLayout->addWidget(lmLabel.get(),2,0);
    formLayout->addWidget(lmLineEdit.get(),2,1);
    formLayout->addWidget(lmBrowseButton.get(),2,2);
    formLayout->addWidget(dictLabel.get(),3,0);
    formLayout->addWidget(dictLineEdit.get(),3,1);
    formLayout->addWidget(dictBrowseButton.get(),3,2);
    formLayout->addWidget(jsgfLabel.get(),4,0);
    formLayout->addWidget(jsgfLineEdit.get(),4,1);
    formLayout->addWidget(jsgfBrowseButton.get(),4,2);
    formLayout->addWidget(rsbLabel.get(),5,0);
    formLayout->addWidget(rsbLineEdit.get(),5,1);
    formLayout->addWidget(bestPathCeckBox.get(),5,2);
    formLayout->addWidget(loadButton.get(),5,3);    
    this->setLayout(formLayout.get());
    QObject::connect (hmmBrowseButton.get(), SIGNAL ( clicked() ), this, SLOT ( hmmBrowse() ));
    QObject::connect (lmBrowseButton.get(), SIGNAL ( clicked() ), this, SLOT ( lmBrowse() ));
    QObject::connect (dictBrowseButton.get(), SIGNAL ( clicked() ), this, SLOT ( dictBrowse() ));
    QObject::connect (jsgfBrowseButton.get(), SIGNAL (clicked() ), this, SLOT (jsgfBrowse() ));
    QObject::connect (loadButton.get(), SIGNAL ( clicked() ), this, SLOT ( loadPS() ));
}


pocketSphinxWidget::pocketSphinxWidget(int id) 
{
    myId = id;
    setupUI(PSW_DEFAULT_NAME,PSW_HMM_DEFAULT,PSW_LM_DEFAULT,PSW_DICT_DEFAULT,PSW_JSGF_DEFAULT,PSW_RSB_SCOPE_DEFAULT,PSW_BESTPATH_DEFAULT);
}


pocketSphinxWidget::pocketSphinxWidget(int id, boost::shared_ptr<PocketSphinxInstanceConfig> myPSIConfig) : myId(id) {
    setupUI(myPSIConfig->GetName(),myPSIConfig->GetHmmDir(),myPSIConfig->GetLmDir(),myPSIConfig->GetDictDir(),myPSIConfig->GetJsgfDir(),myPSIConfig->GetRsbScope(),myPSIConfig->IsBestpath());        
}
pocketSphinxWidget::~pocketSphinxWidget() {
    QObject::disconnect(hmmBrowseButton.get(), SIGNAL( clicked() ), this, SLOT( hmmBrowse() ));
    QObject::disconnect(lmBrowseButton.get(), SIGNAL( clicked() ), this, SLOT( lmBrowse() ));
    QObject::disconnect(dictBrowseButton.get(), SIGNAL( clicked() ), this, SLOT( dictBrowse() ));
    QObject::disconnect(jsgfBrowseButton.get(), SIGNAL( clicked() ), this, SLOT( jsgfBrowse() ));
    QObject::disconnect(loadButton.get(), SIGNAL( clicked() ), this, SLOT( loadPS() ));
}

void
pocketSphinxWidget::hmmBrowse()
{
    QString dirname = QFileDialog::getExistingDirectory(this,tr("HMM Directory"),hmmLineEdit->text(),QFileDialog::ShowDirsOnly);
    if (dirname.compare("") != 0)
        hmmLineEdit->setText(dirname);
}

void
pocketSphinxWidget::lmBrowse()
{
    QString lmname = QFileDialog::getOpenFileName(this,tr("Language Model File"),lmLineEdit->text(),tr("Language Models (*.DMP)"));
    if (lmname.compare("") != 0)
        lmLineEdit->setText(lmname);
}

void
pocketSphinxWidget::dictBrowse()
{
    QString dictname = QFileDialog::getOpenFileName(this,tr("Language Model File"),dictLineEdit->text(),tr("Dictionary (*.dic)"));
    if (dictname.compare("") != 0)
        dictLineEdit->setText(dictname);
}

void
pocketSphinxWidget::jsgfBrowse()
{
    QString jsgfName = QFileDialog::getOpenFileName(this,tr("Java Speech Grammar Format"), jsgfLineEdit->text(), tr("JSGF (*.jsgf)"));
    if (jsgfName.compare("") != 0)
        jsgfLineEdit->setText(jsgfName);
}

void
pocketSphinxWidget::loadPS()
{
    if (psw.use_count() < 1)
    {
        try {
            bool bestpath = bestPathCeckBox->isChecked();
                psw = boost::make_shared<PocketSphinxWrapper>(nameLineEdit->text().toStdString(), hmmLineEdit->text().toStdString(),lmLineEdit->text().toStdString(),dictLineEdit->text().toStdString(),jsgfLineEdit->text().toStdString(),bestpath);                
        }
        catch (PocketSphinxInitException &e)
        {
                QString message ("Could initialize Pocketsphinx: ");
                message.append(e.what());
                errorMessageDialog->showMessage(message);
                return;
        }
        myPublisher = boost::make_shared<publishRsb>(rsbLineEdit->text().toStdString());
        if (myRemoteReceiver.use_count() < 1)
        {
            boost::filesystem::path myRsbPath = rsbLineEdit->text().toStdString();
            myRsbPath.remove_filename();            
            std::string scope ("/request");
            scope.append(myRsbPath.string());
            //scope.append("/commands");
            myRemoteReceiver = boost::make_shared<rsbRemoteReceiver>(scope);
        }
        connect (psw.get(), SIGNAL ( xmlResult (QString) ), myPublisher.get(), SLOT ( publishMessage (QString) ) );
        connect (psw.get(), SIGNAL ( uttResult (QString) ), this, SLOT ( gotUttResult(QString) ) );
        connect (myRemoteReceiver.get(), SIGNAL (remoteDisable() ), psw.get(), SLOT (disableListening() ) );
        connect (myRemoteReceiver.get(), SIGNAL (remoteEnable() ), psw.get(), SLOT (enableListening() ) );
        connect (this, SIGNAL (uttStart() ), psw.get(), SLOT (uttStart() ) );
        connect (this, SIGNAL (uttEnd() ), psw.get(), SLOT (uttEnd() ) );
        connect (this, SIGNAL (uttDiscard() ), psw.get(), SLOT (uttDiscard() ));
        connect (this, SIGNAL (pushChunk (short*, unsigned int) ), psw.get(), SLOT (pushChunk (short*, unsigned int) ) );
        name = nameLineEdit->text();
        rsbScope = rsbLineEdit->text();
        groupBoxLayout = boost::make_shared<QHBoxLayout>();
        //finalGroupBox = boost::make_shared<QGroupBox>(nameLineEdit->text());
        QString infoString;
        if (0 == rsbScope.compare(""))
            infoString  = "PocketSphinx <b>" + name + "</b> loaded successfully";
        else
            infoString = QString ("PocketSphinx <b>"+name+"</b> loaded successfully publishing on scope: <b>").append(rsbScope).append("</b>");
        psInfoLabel = boost::make_shared<QLabel>(infoString);
        groupBoxLayout->addWidget(psInfoLabel.get());
        //finalGroupBox->setLayout(groupBoxLayout.get());
        formLayout->removeWidget(nameLabel.get());
        formLayout->removeWidget(nameLineEdit.get());
        formLayout->removeWidget(hmmLabel.get());
        formLayout->removeWidget(hmmLineEdit.get());
        formLayout->removeWidget(hmmBrowseButton.get());
        formLayout->removeWidget(lmLabel.get());
        formLayout->removeWidget(lmLineEdit.get());
        formLayout->removeWidget(lmBrowseButton.get());
        formLayout->removeWidget(dictLabel.get());
        formLayout->removeWidget(dictLineEdit.get());
        formLayout->removeWidget(dictBrowseButton.get());
        formLayout->removeWidget(jsgfLabel.get());
        formLayout->removeWidget(jsgfLineEdit.get());
        formLayout->removeWidget(jsgfBrowseButton.get());
        formLayout->removeWidget(rsbLabel.get());
        formLayout->removeWidget(rsbLineEdit.get());
        formLayout->removeWidget(bestPathCeckBox.get());
        formLayout->removeWidget(loadButton.get());
        nameLabel.reset();
        nameLineEdit.reset();
        hmmLabel.reset();
        hmmLineEdit.reset();
        hmmBrowseButton.reset();
        lmLabel.reset();
        lmLineEdit.reset();
        lmBrowseButton.reset();
        dictLabel.reset();
        dictLineEdit.reset();
        dictBrowseButton.reset();
        jsgfBrowseButton.reset();
        jsgfLabel.reset();
        jsgfLineEdit.reset();
        rsbLabel.reset();
        rsbLineEdit.reset();
        bestPathCeckBox.reset();
        loadButton.reset();
        formLayout->addLayout(groupBoxLayout.get(),0,0);
        setLayout(formLayout.get());
        emit psLoaded(myId);
    }
}
bool
pocketSphinxWidget::isLoaded()
{
    return (psw.use_count());
}

void
pocketSphinxWidget::gotPushChunk(short* chunk, unsigned int size)
{
    emit pushChunk (chunk,size);
}

void
pocketSphinxWidget::gotUttEnd()
{
    emit uttEnd();
}

void
pocketSphinxWidget::gotUttStart()
{
    emit uttStart();
}

void
pocketSphinxWidget::gotUttResult(QString result)
{
    emit uttResult(result);
}

void
pocketSphinxWidget::gotUttDiscard()
{
    emit uttDiscard();
}

QString
pocketSphinxWidget::getName()
{
    return name;
}

void
pocketSphinxWidget::acceptExternalInput(QString s)
{
    psw->processExternalInput(s);
}

boost::shared_ptr<rsbRemoteReceiver>
pocketSphinxWidget::getRemoteReceiver()
{
    return myRemoteReceiver;
}