/* 
 * File:   pocketSphinxWidget.h
 * Author: mschoepf
 *
 * Created on May 14, 2013, 5:04 PM
 */

#ifndef POCKETSPHINXWIDGET_H
#define	POCKETSPHINXWIDGET_H

#include <fstream>
#include <QApplication>
#include <QLabel>
#include <QLayout>
#include <QPushButton>
#include <QMainWindow>
#include <QTextEdit>
#include <QSpinBox>
#include <QMessageBox>
#include <QErrorMessage>
#include <QCheckBox>
#include <QLineEdit>
#include <QGridLayout>
#include <QFileDialog>
#include <QGroupBox>
#include <boost/shared_ptr.hpp>
#include <boost/make_shared.hpp>
#include "PocketSphinxWrapper.h"
#include "publishRsb.h"
#include "rsbRemoteReceiver.h"
#include "defaults.h"
#include "configFileParser.h"





class pocketSphinxWidget : public QWidget{
    
    Q_OBJECT
    
public:
    pocketSphinxWidget(int id);
    pocketSphinxWidget(int id, boost::shared_ptr<PocketSphinxInstanceConfig> myPSIConfig);
    virtual ~pocketSphinxWidget();
    bool isLoaded();
    QString getName();
    boost::shared_ptr<rsbRemoteReceiver> getRemoteReceiver();
private:
    int myId;
    boost::shared_ptr<QGridLayout> formLayout;
    boost::shared_ptr<QLabel> nameLabel;
    boost::shared_ptr<QLineEdit> nameLineEdit;
    boost::shared_ptr<QHBoxLayout> groupBoxLayout;
    boost::shared_ptr<QGroupBox> finalGroupBox;
    boost::shared_ptr<QLabel> psInfoLabel;
    boost::shared_ptr<QHBoxLayout> finalLayout;
    boost::shared_ptr<QLabel> hmmLabel;
    boost::shared_ptr<QLineEdit> hmmLineEdit;
    boost::shared_ptr<QPushButton> hmmBrowseButton;
    boost::shared_ptr<QLabel> lmLabel;
    boost::shared_ptr<QLineEdit> lmLineEdit;
    boost::shared_ptr<QPushButton> lmBrowseButton;
    boost::shared_ptr<QLabel> dictLabel;
    boost::shared_ptr<QLineEdit> dictLineEdit;
    boost::shared_ptr<QPushButton> dictBrowseButton;
    boost::shared_ptr<QLabel> jsgfLabel;
    boost::shared_ptr<QLineEdit> jsgfLineEdit;
    boost::shared_ptr<QPushButton> jsgfBrowseButton;
    boost::shared_ptr<QLabel> rsbLabel;
    boost::shared_ptr<QLineEdit> rsbLineEdit;
    boost::shared_ptr<QPushButton> loadButton;
    boost::shared_ptr<QPushButton> unLoadButton;
    boost::shared_ptr<QCheckBox> bestPathCeckBox;
    boost::shared_ptr<QErrorMessage> errorMessageDialog;
    boost::shared_ptr<PocketSphinxWrapper> psw;
    boost::shared_ptr<publishRsb> myPublisher;
    QString name;
    QString rsbScope;
    static boost::shared_ptr<rsbRemoteReceiver> myRemoteReceiver;
    void setupUI(std::string name, std::string hmm, std::string lm, std::string dict, std::string jsgf, std::string rsb, bool bestpath);
public slots:
    void hmmBrowse();
    void lmBrowse();
    void dictBrowse();
    void jsgfBrowse();
    void loadPS();
    void gotPushChunk (short* chunk, unsigned int size);
    void gotUttStart();
    void gotUttEnd();
    void gotUttDiscard();
    void gotUttResult (QString result);
    void acceptExternalInput (QString);
    
    
signals:
    void uttResult (QString result);
    void pushChunk (short* chunk, unsigned int size);
    void uttStart();
    void uttDiscard();
    void uttEnd();
    void psLoaded(int id);
};

#endif	/* POCKETSPHINXWIDGET_H */

