/*
 * PocketSphinxWrapper.cpp
 *
 *  Created on: Apr 16, 2012
 *      Author: mschoepf
 */

#include "AlsaMicGrabber.h"


#include <boost/thread/pthread/condition_variable_fwd.hpp>
#include <boost/algorithm/string/replace.hpp>

#include "PocketSphinxWrapper.h"
#include "PocketSphinxWidget.h"




PocketSphinxWrapper::~PocketSphinxWrapper() {
	ps_free(pocketSphinx);
	cmd_ln_free_r(cmdLineConfig);  //cmd_ln_free_r unfortunately is broken
}

PocketSphinxWrapper::PocketSphinxWrapper(std::string name, std::string hmm, std::string lm, std::string dict, std::string jsgf, bool bestpath) : name(name), listening(true), dataReady(false) {
    std::vector<std::string> arguments;    
    int noOfArguments = 0;
    if (hmm.compare("") != 0) {
        noOfArguments++;
        arguments.push_back("-hmm");
        arguments.push_back(hmm);
    }
    if (lm.compare("") != 0) {
        noOfArguments++;
        arguments.push_back("-lm");
        arguments.push_back(lm);
    }
    if (dict.compare("") != 0) {
        noOfArguments++;
        arguments.push_back("-dict");
        arguments.push_back(dict);
    }
    if (jsgf.compare("") != 0) {
        noOfArguments++;
        arguments.push_back("-jsgf");
        arguments.push_back(jsgf);
        haveJsgf = true;
    }
    if (!bestpath)
    {
        arguments.push_back("-bestpath");
        arguments.push_back("no");
        noOfArguments++;
    }
    switch (noOfArguments) {
        case 0: cmdLineConfig = cmd_ln_init(NULL, ps_args(), 1, NULL);
            break;
        case 1: cmdLineConfig = cmd_ln_init(NULL, ps_args(), 1, arguments.at(0).c_str(), arguments.at(1).c_str(), NULL);
            break;
        case 2: cmdLineConfig = cmd_ln_init(NULL, ps_args(), 1, arguments.at(0).c_str(), arguments.at(1).c_str(), arguments.at(2).c_str(), arguments.at(3).c_str(), NULL);
            break;
        case 3: cmdLineConfig = cmd_ln_init(NULL, ps_args(), 1, arguments.at(0).c_str(), arguments.at(1).c_str(), arguments.at(2).c_str(), arguments.at(3).c_str(), arguments.at(4).c_str(), arguments.at(5).c_str(), NULL);
            break;
        case 4: cmdLineConfig = cmd_ln_init(NULL, ps_args(), 1, arguments.at(0).c_str(), arguments.at(1).c_str(), arguments.at(2).c_str(), arguments.at(3).c_str(), arguments.at(4).c_str(), arguments.at(5).c_str(), arguments.at(6).c_str(), arguments.at(7).c_str(), NULL);
            break;
        case 5: cmdLineConfig = cmd_ln_init(NULL, ps_args(), 1, arguments.at(0).c_str(), arguments.at(1).c_str(), arguments.at(2).c_str(), arguments.at(3).c_str(), arguments.at(4).c_str(), arguments.at(5).c_str(), arguments.at(6).c_str(), arguments.at(7).c_str(), arguments.at(8).c_str(), arguments.at(9).c_str(), NULL);
            break;
        default:
            cmdLineConfig = cmd_ln_init(NULL, ps_args(), 1, NULL);
    }

    if (cmdLineConfig == NULL) {
        throw PocketSphinxInitException("Could not initialize Commandline Arguments");
    }
    pocketSphinx = ps_init(cmdLineConfig);
    if (pocketSphinx == NULL) {
        throw PocketSphinxInitException("Could not initialize Pocketsphinx, maybe arguments are erroneous");
    }
    if (haveJsgf) {
        if (0 != jsgf.compare("")) {
            std::ifstream input(jsgf.c_str(), std::ifstream::in);
            if (!input.is_open()) {
                std::cerr << "Could not open file: " << jsgf.c_str() << std::endl;
                std::string message ("Could not open :");
                message.append(jsgf);
                throw PocketSphinxInitException(message.c_str());                
            }
            std::string storage; // We will read the contents here.
            input.unsetf(std::ios::skipws); // No white space skipping!
            std::copy(
                    std::istream_iterator<char>(input),
                    std::istream_iterator<char>(),
                    std::back_inserter(storage));
            jsgfParserRuleGenerator<std::string::const_iterator> myJsgfParser;
            std::string::const_iterator iter = storage.begin();
            std::string::const_iterator end = storage.end();
            bool success = qi::phrase_parse(iter, end, myJsgfParser, ascii::space, myGrammar);
            if (success && iter == end) {
                std::cout << "-------------------------\n";
                std::cout << "Parsing succeeded\n";
                std::cout << "-------------------------\n";
            } else {
                std::string::const_iterator some = iter + 30;
                std::string context(iter, (some > end) ? end : some);
                std::string message ("JSGF Parsing failed, stopped at: '" + context + "'");
                throw PocketSphinxInitException(message);                
            }
            if (myJsgfParser.sanitizeCheckGrammar(myGrammar, true)) {
                std::cout << "JSGF check success" << std::endl;
            } else {
                std::cout << "JSGF check FAILED" << std::endl;
                throw PocketSphinxInitException("Could not generate JSGF Parser");
            }
            std::cout << "My grammars Name is: " << myGrammar.name << " and the first rules name is " << myGrammar.rules.at(0).name << std::endl;
            if (myJsgfParser.generateRules(myGrammar)) {
                std::cout << "JSGF rules generated" << std::endl;
            } else {
                std::cout << "JSGF rule generation FAILED" << std::endl;
                throw PocketSphinxInitException("JSGF Rule Generation failed");
            }
            std::vector<jsgfParserTypes::ruleTPtr> myRules = myJsgfParser.getRules();
            myParser.reset(new dynamicParser<std::string::iterator>(myRules));
        }
    }
    analyzeThread = boost::make_shared<boost::thread>(&PocketSphinxWrapper::RecognizeFromMicrophone, this);
}

std::string 
PocketSphinxWrapper::ExtractTerminalFromGrammar (const std::string &xmlString)
{
    qi::rule<std::string::const_iterator, std::string(), qi::locals<std::string>, ascii::space_type> xml;
    qi::rule<std::string::const_iterator, std::string(), ascii::space_type> node;
    qi::rule<std::string::const_iterator, std::string(), ascii::space_type> text;
    qi::rule<std::string::const_iterator, std::string(), ascii::space_type> start_tag;
    qi::rule<std::string::const_iterator, void(std::string), ascii::space_type> end_tag;
    text %= qi::lexeme[+(qi::char_ - '<')];
    node %= xml | text;
    start_tag %=
            '<'
            >> !qi::lit('/')
            >> qi::lexeme[+(qi::char_ - '>')]
            >> '>'
            ;

    end_tag =
            "</"
            >> qi::string(qi::_r1)
            >> qi::lit('>')
            ;

    xml =
            start_tag[qi::_a = qi::_1]
            >> *node [qi::_val = qi::_1]
            >> end_tag(px::val("nonterminal"))
            ;       
    std::string::const_iterator iter = xmlString.begin();
    std::string::const_iterator end = xmlString.end();
    std::string result;
    bool success = qi::phrase_parse(iter, end, xml, ascii::space, result);
    if (success && iter == end) {
        return result;
    }
    std::cerr << "Internal Error occurred here: PocketSphinxWrapper::ExtractTerminalFromGrammar" << std::endl;
    abort();
    return result;
}

std::string
PocketSphinxWrapper::ReplaceTerminalWithId (const std::string &xmlString, unsigned int &startid) {
    std::string result = xmlString;
    std::string terminal = ExtractTerminalFromGrammar(xmlString);
    std::cout << "DEBUG xmlString  " << xmlString << std::endl;
    std::cout << "DEBUG terminal = " << terminal << std::endl;
    std::string replaceMe = ">";
    replaceMe.append(terminal);
    replaceMe.append("<");
    std::cout << "DEBUG replaceme = " << replaceMe << std::endl;
    size_t n = 1 + std::count(terminal.begin(), terminal.end(), ' ');
    std::stringstream s;
    s << ">";
    for (size_t i=0; i < n; i++)
        s << "<terminal refid=\"" << startid++ <<"\"/>";
    s << "<";
    boost::replace_first(result, replaceMe,s.str());
    return result;
}

QString
PocketSphinxWrapper::MakeXml (std::string hypothese, int score, std::vector<std::string> grammartree)
{
    std::stringstream xml;
    boost::posix_time::ptime time_t_epoch(boost::gregorian::date(1970,1,1));    
    boost::posix_time::ptime now = boost::posix_time::microsec_clock::local_time();
    boost::posix_time::time_duration timestamp = now - time_t_epoch;
    std::vector<std::string> hypVector;
    boost::split(hypVector,hypothese, boost::is_any_of("\t "));
    xml << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>";
    xml << "<speech_hyp generator=\"pocketsphinxAdapter\" origin=\"" << name << "\" stable=\"1\">";
    xml << "<TIMESTAMP>" << timestamp.total_milliseconds() << "</TIMESTAMP>";
    xml << "<seq>";
    unsigned int id=0;
    for (std::vector<std::string>::const_iterator it=hypVector.begin();it != hypVector.end();it++)
    {
        xml << "<part begin=\"0\" end=\"1\" id=\"" << id << "\" type=\"word\">" << *it << "</part>";
        xml << "<score acoustic=\"" << score << "\" combined=\"" << score << "\" id=\"" << id++ << "\" lm=\"" << score << "\" />";
    }
    xml << "</seq>";
    xml << "<grammartree cancel=\"0\" fault=\"0\" skip=\"0\">";
    id = 0;
    for (std::vector<std::string>::const_iterator it=grammartree.begin();it != grammartree.end();it++)        
        xml <<  ReplaceTerminalWithId (*it,id);
    xml << "</grammartree>";
    xml << "</speech_hyp>";
    return QString(xml.str().c_str());    
}
void
PocketSphinxWrapper::processExternalInput(QString input) {
    std::string myInput = input.toStdString();
    QString result;
    if ((myInput.compare("") != 0)) {
        if (haveJsgf) {
            std::string::iterator ststart = myInput.begin();
            std::string::iterator stend = myInput.end();
            std::vector<std::string> attr;
            bool dynSucc = boost::spirit::qi::phrase_parse(ststart, stend, (*myParser), boost::spirit::qi::ascii::space, attr);
            if (dynSucc) {
                if (ststart != stend) {
                    std::string partly(myInput.begin(), stend);
                    partly.append(" <- parsed PARTLY, Leftover: ");
                    partly.append(std::string().assign(ststart, stend));
                    std::cout << "'" << myInput << "' parsed PARTLY successfully " << std::endl;
                    std::cout << "Leftover: '" << std::string().assign(ststart, stend) << "'" << std::endl;
                    result = QString("###-> ");
                    result.append(partly.c_str());
                } else {
                    std::cout << "'" << myInput << "' parsed successfully " << std::endl;
                    result = input;
                    QString xml = MakeXml(myInput, 100, attr);
                    std::cout << xml.toStdString() << std::endl;
                    std::cout << "Resultvector: ";
                    for (std::vector<std::string>::iterator it = attr.begin(); it != attr.end(); it++) {                        
                        std::cout << "[" << *it << "] ";
                    }
                    std::cout << std::endl;
                    std::cout << xml.toStdString() << std::endl;
                    emit xmlResult(xml);
                }
            } else {
                std::cout << "'" << myInput << "' not part of the grammar" << std::endl;
                myInput.append(" <- Not Part of the Grammar");
                result = QString("%%% -> ");
                result.append(myInput.c_str());
            }            
        }        
    }
    emit uttResult(result);
}
void
PocketSphinxWrapper::RecognizeFromMicrophone()
{
    std::cerr << "In Recognize" << std::endl;
    int rv;
    char const *hyp, *uttid;
    int score;
    QString utterance;
    while (true)
    {
        std::vector<AlsaSoundChunkT> nextChunkToProcess;
        {
        boost::unique_lock<boost::mutex> lock (queueMutex);
        std::cerr << "Got Lock, waiting for condition" << std::endl;
        while (0 == myQueue.size())
            condWaitEndUtt.wait(lock);
        nextChunkToProcess = myQueue.front();
        myQueue.pop();
        }
        rv = ps_start_utt(pocketSphinx,NULL);
        if (rv < 0)
            std::cerr << "ps_start_utt gave error: " << rv << std::endl;
        std::cout << "Vector size: " << nextChunkToProcess.size() << std::endl;
        for (std::vector<AlsaSoundChunkT>::const_reverse_iterator rit=nextChunkToProcess.rbegin();rit != nextChunkToProcess.rend();rit++) {
            rv = ps_process_raw(pocketSphinx,(short*) &rit->soundchunk[0],ALSAMICGRABBER_SAMPLESIZE,0,0);            
            if (rv < 0)
              std::cerr << "ps_process_raw gave error: " << rv << std::endl;
        }
        rv = ps_end_utt(pocketSphinx);
        if (rv < 0)
            std::cerr << "ps_end_utt gave error: " << rv << std::endl;
        /*ps_nbest_t* myNbestT = ps_nbest(pocketSphinx, 0, -1, NULL, NULL);
        int n = 1;
        while (NULL != ps_nbest_next(myNbestT)) {            
            ps_seg_t *seg;
            hyp = ps_nbest_hyp(myNbestT, &score);
            printf("NBEST %d: %s (%d)\n", n, hyp, score);
            for (seg = ps_nbest_seg(myNbestT, &score); seg;
                    seg = ps_seg_next(seg)) {
                char const *word;
                int sf, ef;

                word = ps_seg_word(seg);
                ps_seg_frames(seg, &sf, &ef);
                printf("%s %d %d\n", word, sf, ef);
            }
            if (n == 10)
                break;
            ++n;            
        }
        ps_nbest_free(myNbestT);
*/
        hyp = ps_get_hyp(pocketSphinx,&score,&uttid);
        dataReady = false;
        if ((hyp != NULL) && (strcmp(hyp, "") != 0)) {
            std::cerr << "OUTPUT: '" << hyp << "'" << std::endl;
            std::cerr << "Score: " << score << std::endl;
            std::string st(hyp);
            if (haveJsgf) {
                std::string::iterator ststart = st.begin();
                std::string::iterator stend = st.end();
                std::vector<std::string> attr;
                bool dynSucc = boost::spirit::qi::phrase_parse(ststart, stend, (*myParser), boost::spirit::qi::ascii::space, attr);
                if (dynSucc) {
                    if (ststart != stend) {
                        std::cout << "'" << st << "' parsed PARTLY successfully " << std::endl;
                        std::cout << "Leftover: '" << std::string().assign(ststart, stend) << "'" << std::endl;
                    } else
                        std::cout << "'" << st << "' parsed successfully " << std::endl;
                    QString xml = MakeXml(st,score,attr);
                    std::cout << xml.toStdString() << std::endl;
                    std::cout << "Resultvector: ";
                    for (std::vector<std::string>::iterator it = attr.begin(); it != attr.end(); it++) {
                        //st.append("[" + (*it) + "] ");
                        std::cout << "[" << *it << "] ";
                    }
                    std::cout << std::endl;
                    std::cout << xml.toStdString() << std::endl;
                    emit xmlResult(xml);
                } else
                    std::cout << "'" << st << "' not part of the grammar" << std::endl;
            }            
            utterance = QString(st.c_str());
        } else
        {
            std::cerr << "OUTPUT: <NULL>" << std::endl;
            utterance = QString("<NULL>");
        }
        
        emit uttResult (utterance);
    }
}

void
PocketSphinxWrapper::pushChunk(short* chunk, unsigned int size)
{
    int rv;
    AlsaSoundChunkT asc;    
    memcpy(asc.soundchunk,chunk,ALSAMICGRABBER_BUFFERSIZE);        
    myBuffer.push_back(asc);
    boost::unique_lock<boost::mutex> lock (processOnlyOne);
    countChunks++;
}

void
PocketSphinxWrapper::uttStart()
{
    int rv;    
    std::cerr << "utt start" << std::endl;
    if (!listening)
        return;
    boost::unique_lock<boost::mutex> lock (processOnlyOne);
    countChunks = 2; //We take just a little before we hit the level
}

void
PocketSphinxWrapper::uttEnd()
{
    int rv;
    if (!listening)
        return;
    std::vector<AlsaSoundChunkT> temp;
    {
        std::cerr << "utt end, cout at " << countChunks << "which is: " << countChunks*static_cast<int>(ALSAMICGRABBER_SAMPLERATE/ALSAMICGRABBER_SAMPLESIZE) << " ms" << std::endl;
        boost::unique_lock<boost::mutex> lock (processOnlyOne);
        temp = myBuffer.getLastNChunks(countChunks);
    }
    {
        boost::unique_lock<boost::mutex> lock(queueMutex);
        myQueue.push(temp);
    }
    condWaitEndUtt.notify_one();
}

void
PocketSphinxWrapper::uttDiscard()
{
    int rv;
    std::cerr << "utt discard" << std::endl;
    boost::unique_lock<boost::mutex> lock (processOnlyOne);    
    countChunks = 0;
}

void
PocketSphinxWrapper::enableListening()
{
    listening = true;
    std::cout << "Listening enabled" << std::endl;
}

void
PocketSphinxWrapper::disableListening()
{
    uttDiscard();
    std::cout << "listening disabled" << std::endl;
    listening = false;
    boost::unique_lock<boost::mutex> lock (queueMutex);
    while (!myQueue.empty())
        myQueue.pop();    
}