/*
 * PocketSphinxWrapper.h
 *
 *  Created on: Apr 16, 2012
 *      Author: mschoepf
 */

#ifndef POCKETSPHINXWRAPPER_H_
#define POCKETSPHINXWRAPPER_H_



#include <Qt/qobject.h>
#include <string>
#include <sstream>
#include <algorithm>
#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>
#include <boost/make_shared.hpp>
#include <boost/thread/condition_variable.hpp>
#include <boost/spirit/include/qi.hpp>
#include <boost/algorithm/string.hpp>
#include <pocketsphinx/pocketsphinx.h>
#include <sphinxbase/glist.h>
#include <sphinxbase/jsgf.h>
#include <iostream>
#include <queue>
#include "PocketSphinxInitException.h"
#include "dynamicParser.h"
#include "JsgfParser.h"
#include "dataRingBuffer.h"
#include "AlsaMicGrabber.h"

class PocketSphinxWrapper : public QObject {
    
    Q_OBJECT
    
private:
	ps_decoder_t* pocketSphinx;
	cmd_ln_t* cmdLineConfig;
        bool listening;
        bool dataReady;
        bool haveJsgf;
        std::string name;
        boost::condition_variable condWaitEndUtt;
        boost::mutex processOnlyOne;
        boost::shared_ptr<boost::thread> analyzeThread;
        boost::shared_ptr<dynamicParser<std::string::iterator> > myParser;
        jsgfParserTypes::GrammarT myGrammar;
        std::vector<jsgfParserTypes::ruleTPtr> myRules;
        std::string ExtractTerminalFromGrammar (const std::string &xml);
        std::string ReplaceTerminalWithId (const std::string &xmlString, unsigned int &startid);
        dataRingBuffer myBuffer;
        size_t countChunks;
        boost::mutex queueMutex;
        std::queue<std::vector<AlsaSoundChunkT> > myQueue;        
public:	
        PocketSphinxWrapper(std::string name, std::string hmm, std::string lm, std::string dict, std::string jsgf,bool bestpath);        
	void RecognizeFromMicrophone();
	virtual ~PocketSphinxWrapper();
        QString MakeXml(std::string hypothese, int score, std::vector<std::string> grammartree);
        void processExternalInput(QString input);        
public slots:
    void pushChunk (short* chunk, unsigned int size);
    void uttStart();
    void uttEnd();
    void uttDiscard();
    void enableListening();
    void disableListening();
    
signals:
    void uttResult (QString result);
    void xmlResult (QString result);
            
};

#endif /* POCKETSPHINXWRAPPER_H_ */
