#include <qt4/Qt/qlayout.h>
#include <qt4/Qt/qmessagebox.h>

#include "PocketsphinxAdapter.h"
#include "configFileParser.h"





pocketsphinxAdapterMainWindow::pocketsphinxAdapterMainWindow(QWidget* parent) : QMainWindow (parent), connectedToRemote(false)
{
	maxDb = averageDb = 0.0;
        psInstanceCount = 0;
	baseFrame = boost::make_shared<QFrame>(this);
	baseFrame->setGeometry (0, 0, 1024, 800);
	vBoxMainLayout = boost::make_shared<QVBoxLayout>(baseFrame.get());
        hBoxAlsaControlLayout = boost::make_shared<QHBoxLayout>();
        alsaDeviceLabel = boost::make_shared<QLabel> ("Alsa Device: ");
        alsaDeviceLineEdit = boost::make_shared<QLineEdit>(QString("plughw:0,0"));
        openAlsaDeviceCheckBox = boost::make_shared<QCheckBox>("Open Alsa Device");
        publishStatusCheckBox = boost::make_shared<QCheckBox>("Publish to RSB");
        publishStatusCheckBox->setChecked(true);
        hBoxAlsaControlLayout->addWidget(alsaDeviceLabel.get());
        hBoxAlsaControlLayout->addWidget(alsaDeviceLineEdit.get());
        hBoxAlsaControlLayout->addWidget(openAlsaDeviceCheckBox.get());
        hBoxAlsaControlLayout->addWidget(publishStatusCheckBox.get());
        vBoxMainLayout->addLayout(hBoxAlsaControlLayout.get());
	equalizerWidget = boost::make_shared<EqualizerColorWidget>();
	vBoxMainLayout->addWidget(equalizerWidget.get());
	dbLabel = boost::make_shared<QLabel>(QString ("Level:  0.0 db, LTAverage:  0.0 db, STAverage:  0.0 db, Max:  0.0 db Phase: sleeping"));
	vBoxMainLayout->addWidget(dbLabel.get());
        simpleSegmenterWidget = boost::make_shared<simpleDoubleThresholdSegmenter>();
        vBoxMainLayout->addWidget(simpleSegmenterWidget.get());
	wellLabel = boost::make_shared<QLabel>("Well, all I can hear is ....: ");
	vBoxMainLayout->addWidget(wellLabel.get());
        psResultsTab = boost::make_shared<QTabWidget>();
        //psResultsTab->sizeHint(QSize(500,500));
        vBoxMainLayout->addWidget(psResultsTab.get());
        addPWidget = boost::make_shared<QPushButton>("Add Pocketsphinx Dialog");
        pWidgetLayout = boost::make_shared<QVBoxLayout>();
        vBoxMainLayout->addWidget(addPWidget.get());
        vBoxMainLayout->addLayout(pWidgetLayout.get());
	quitButton = boost::make_shared<QPushButton> ("Quit");
	vBoxMainLayout->addWidget(quitButton.get());        
	connect(quitButton.get(), SIGNAL ( clicked()), this, SLOT ( quit()));
        connect(openAlsaDeviceCheckBox.get(), SIGNAL ( stateChanged (int) ), this, SLOT ( handleAlsa (int) ));
        connect(addPWidget.get(), SIGNAL (clicked() ), this, SLOT (addPocketSphinxWidget() ) );
        /*
        QObject::connect(&amg, SIGNAL ( updated_dB_and_clipped(float, bool) ), &w, SLOT ( update_eq_only(float, bool) ) );*/
        this->setLayout(vBoxMainLayout.get());
}

pocketsphinxAdapterMainWindow::pocketsphinxAdapterMainWindow(boost::shared_ptr<PocketSphinxGlobalConfig> myConfig, QWidget* parent) : QMainWindow(parent), connectedToRemote(false) {
    maxDb = averageDb = 0.0;
    psInstanceCount = 0;
    baseFrame = boost::make_shared<QFrame>(this);
    baseFrame->setGeometry(0, 0, 1024, 800);
    vBoxMainLayout = boost::make_shared<QVBoxLayout>(baseFrame.get());
    hBoxAlsaControlLayout = boost::make_shared<QHBoxLayout>();
    alsaDeviceLabel = boost::make_shared<QLabel> ("Alsa Device: ");
    alsaDeviceLineEdit = boost::make_shared<QLineEdit>(QString(myConfig->GetAlsaDevice().c_str()));
    openAlsaDeviceCheckBox = boost::make_shared<QCheckBox>("Open Alsa Device");
    publishStatusCheckBox = boost::make_shared<QCheckBox> ("Publish to RSB");
    publishStatusCheckBox->setChecked(true);
    hBoxAlsaControlLayout->addWidget(alsaDeviceLabel.get());
    hBoxAlsaControlLayout->addWidget(alsaDeviceLineEdit.get());
    hBoxAlsaControlLayout->addWidget(openAlsaDeviceCheckBox.get());
    hBoxAlsaControlLayout->addWidget(publishStatusCheckBox.get());
    vBoxMainLayout->addLayout(hBoxAlsaControlLayout.get());
    equalizerWidget = boost::make_shared<EqualizerColorWidget>();
    vBoxMainLayout->addWidget(equalizerWidget.get());
    dbLabel = boost::make_shared<QLabel>(QString("Level:  0.0 db, LTAverage:  0.0 db, STAverage:  0.0 db, Max:  0.0 db Phase: sleeping"));
    vBoxMainLayout->addWidget(dbLabel.get());
    simpleSegmenterWidget = boost::make_shared<simpleDoubleThresholdSegmenter>(myConfig->GetStartDB(),myConfig->GetUttDB(),myConfig->GetSilence(),myConfig->GetTimeout());
    vBoxMainLayout->addWidget(simpleSegmenterWidget.get());
    wellLabel = boost::make_shared<QLabel>("Well, all I can hear is ....: ");
    vBoxMainLayout->addWidget(wellLabel.get());
    psResultsTab = boost::make_shared<QTabWidget>();
    //psResultsTab->sizeHint(QSize(500,500));
    vBoxMainLayout->addWidget(psResultsTab.get());
    addPWidget = boost::make_shared<QPushButton>("Add Pocketsphinx Dialog");
    pWidgetLayout = boost::make_shared<QVBoxLayout>();
    vBoxMainLayout->addWidget(addPWidget.get());
    vBoxMainLayout->addLayout(pWidgetLayout.get());
    quitButton = boost::make_shared<QPushButton> ("Quit");
    vBoxMainLayout->addWidget(quitButton.get());
    connect(quitButton.get(), SIGNAL(clicked()), this, SLOT(quit()));
    connect(openAlsaDeviceCheckBox.get(), SIGNAL(stateChanged(int)), this, SLOT(handleAlsa(int)));
    connect(addPWidget.get(), SIGNAL(clicked()), this, SLOT(addPocketSphinxWidget()));
    connect(publishStatusCheckBox.get(), SIGNAL( stateChanged(int) ), this, SLOT (gotToggle(int) ) );
    std::vector < boost::shared_ptr<PocketSphinxInstanceConfig> > psConfigVector = myConfig->GetPocketSphinxInstanceVector();
    for (std::vector<boost::shared_ptr<PocketSphinxInstanceConfig> >::iterator it = psConfigVector.begin(); it != psConfigVector.end(); it++) {
        pWidget.push_back(boost::make_shared<pocketSphinxWidget>(psInstanceCount,*it));
        pWidgetLayout->addWidget(pWidget.at(psInstanceCount).get());
        psResultsTextEdit.push_back(boost::make_shared<simpleTextAddField>());
        connect(pWidget.at(psInstanceCount).get(), SIGNAL(psLoaded(int)), this, SLOT(psAddToText(int)));
        psInstanceCount++;
        if ((*it)->IsAutoload())
        {
          pWidget.back()->loadPS();
          boost::this_thread::sleep(boost::posix_time::millisec(500));
        }
    }    
    this->setLayout(vBoxMainLayout.get());
    /*
    QObject::connect(&amg, SIGNAL ( updated_dB_and_clipped(float, bool) ), &w, SLOT ( update_eq_only(float, bool) ) );*/
}


pocketsphinxAdapterMainWindow::~pocketsphinxAdapterMainWindow()
{
	disconnect(quitButton.get(), SIGNAL ( clicked()), this, SLOT ( quit()));
	//delete isrResultsTextEdit;
	//delete equalizerWidget;
	//delete vBoxMainLayout;
	//delete baseFrame;
}

void
pocketsphinxAdapterMainWindow::addPocketSphinxWidget()
{
    pWidget.push_back(boost::make_shared<pocketSphinxWidget>(psInstanceCount));
    pWidgetLayout->addWidget(pWidget.at(psInstanceCount).get());
    psResultsTextEdit.push_back(boost::make_shared<simpleTextAddField>());
    connect (pWidget.at(psInstanceCount).get(), SIGNAL (psLoaded (int) ), this, SLOT (psAddToText(int) ) );
    psInstanceCount++;    
}

void
pocketsphinxAdapterMainWindow::psAddToText(int id)
{
    if (!connectedToRemote)
    {
        boost::shared_ptr<rsbRemoteReceiver> localReceiver = pWidget.at(id)->getRemoteReceiver();
        connect (localReceiver.get(), SIGNAL (remoteEnable() ),  SLOT (setPublishEnable() ) );
        connect (localReceiver.get(), SIGNAL (remoteDisable() ), SLOT (setPublishDisable() ) );
        connect (this, SIGNAL (overwriteEnable() ),localReceiver.get(), SLOT (overwriteEnable() ) );
        connect (this, SIGNAL (overwriteDisable() ), localReceiver.get(), SLOT (overwriteDisable() ) );
    }
    psResultsTab->insertTab(id,psResultsTextEdit.at(id).get(),pWidget.at(id)->getName());
    connect (pWidget.at(id).get(), SIGNAL (uttResult(QString) ), psResultsTextEdit.at(id).get(), SLOT (update_text (QString) ));
    connect (psResultsTextEdit.at(id).get(), SIGNAL (uttInput (QString) ), pWidget.at(id).get(), SLOT (acceptExternalInput (QString) ));
}

void
pocketsphinxAdapterMainWindow::handleAlsa(int state) {
    disconnect(openAlsaDeviceCheckBox.get(), SIGNAL(stateChanged(int)), this, SLOT(handleAlsa(int)));
    if (Qt::Unchecked == state) {
        disconnect(amg.get(), SIGNAL(updated_dB_and_clipped(float, bool) ), this, SLOT(update_eq_only(float, bool)));
        disconnect(amg.get(), SIGNAL(updated_dB_and_clipped(float, bool) ), simpleSegmenterWidget.get(), SLOT(getDBandClipped(float, bool)));        
        update_eq_only(0.0, false);
        for (int i = 0; i < psInstanceCount; i++) {
            if (pWidget.at(i)->isLoaded()) {
                disconnect(amg.get(), SIGNAL(pushGrab(short*, unsigned int)), pWidget.at(i).get(), SLOT(gotPushChunk(short*, unsigned int)));
                //connect (startUttButton, SIGNAL (clicked () ), pWidget, SLOT (gotUttStart() ) );
                //connect (endUttButton, SIGNAL (clicked () ), pWidget, SLOT (gotUttEnd() ) );
                disconnect(simpleSegmenterWidget.get(), SIGNAL(uttStart()), pWidget.at(i).get(), SLOT(gotUttStart()));
                disconnect(simpleSegmenterWidget.get(), SIGNAL(uttEnd()), pWidget.at(i).get(), SLOT(gotUttEnd()));
                disconnect(simpleSegmenterWidget.get(), SIGNAL(uttDiscard()), pWidget.at(i).get(), SLOT(gotUttDiscard()));
            }
        }
        amg.reset();
        openAlsaDeviceCheckBox->setChecked(false);
        connect(openAlsaDeviceCheckBox.get(), SIGNAL(stateChanged(int)), this, SLOT(handleAlsa(int)));
        return;
    }
    try {
        amg = boost::make_shared<AlsaMicGrabber>(alsaDeviceLineEdit->text().toStdString());
    }    catch (AlsaPcmDeviceNotFoundException &e) {
        std::cerr << "Device not found (permissions?): " << alsaDeviceLineEdit->text().toStdString().c_str() << std::endl;
        openAlsaDeviceCheckBox->setChecked(false);
        connect(openAlsaDeviceCheckBox.get(), SIGNAL(stateChanged(int)), this, SLOT(handleAlsa(int)));
        return;
    }
    connect(amg.get(), SIGNAL(updated_dB_and_clipped(float, bool)), this, SLOT(update_eq_only(float, bool)));
    connect(amg.get(), SIGNAL(updated_dB_and_clipped(float, bool)), simpleSegmenterWidget.get(), SLOT(getDBandClipped(float, bool)));
    for (int i = 0; i < psInstanceCount; i++) {
        if (pWidget.at(i)->isLoaded()) {
            connect(amg.get(), SIGNAL(pushGrab(short*, unsigned int)), pWidget.at(i).get(), SLOT(gotPushChunk(short*, unsigned int)));
            //connect (startUttButton, SIGNAL (clicked () ), pWidget, SLOT (gotUttStart() ) );
            //connect (endUttButton, SIGNAL (clicked () ), pWidget, SLOT (gotUttEnd() ) );
            connect(simpleSegmenterWidget.get(), SIGNAL(uttStart()), pWidget.at(i).get(), SLOT(gotUttStart()));
            connect(simpleSegmenterWidget.get(), SIGNAL(uttEnd()), pWidget.at(i).get(), SLOT(gotUttEnd()));
            connect(simpleSegmenterWidget.get(), SIGNAL(uttDiscard()), pWidget.at(i).get(), SLOT(gotUttDiscard()));
        }
    }
    openAlsaDeviceCheckBox->setChecked(true);
    connect(openAlsaDeviceCheckBox.get(), SIGNAL(stateChanged(int)), this, SLOT(handleAlsa(int)));
}

void
pocketsphinxAdapterMainWindow::update_eq_string(float val, int db_start, int db_utt, QString phase)
{
	equalizerWidget->setLevel(val);
	if (phase.compare("sleeping") == 0)
		equalizerWidget->setColor(Qt::green);
	if ((phase.compare("rising") == 0) || (phase.compare("list") == 0))
		equalizerWidget->setColor(Qt::blue);
	if (phase.compare("falling") == 0)
		equalizerWidget->setColor(Qt::yellow);
	std::stringstream s;
	QString qs;
	if (val > maxDb)
		maxDb = val;
	averageDb = 0.95*averageDb  + 0.05*val;
	stAverageDb = 0.75*stAverageDb + 0.25*val;
	s << "Level:  " << std::setw(6) << std::setprecision(4) << std::showpoint << val << " db, LTAv.: " << averageDb << " db, STAv.: " << stAverageDb <<" db, Max:  " << maxDb << " db Start: " << db_start << " db Utt: " << db_utt << "db  Phase: " << phase.toStdString();
	dbLabel->setText(s.str().c_str());
}

void
pocketsphinxAdapterMainWindow::update_eq_only(float val, bool clipped) {
    equalizerWidget->setLevel(val);
    if (clipped)
        equalizerWidget->setColor(Qt::red);
    else
        equalizerWidget->setColor(Qt::green);
    std::stringstream s;
    QString qs;
    if (val > maxDb)
        maxDb = val;
    averageDb = 0.95 * averageDb + 0.05 * val;
    stAverageDb = 0.75 * stAverageDb + 0.25 * val;
    s << "Level:  " << std::setw(6) << std::setprecision(4) << std::showpoint << val << " db, LTAv.: " << averageDb << " db, STAv.: " << stAverageDb << " db, Max:  " << maxDb;
    dbLabel->setText(s.str().c_str());
}

void
pocketsphinxAdapterMainWindow::quit()
{
    if (openAlsaDeviceCheckBox->isChecked())
    {
        switch  (	QMessageBox::warning(
		this, "Do you REALLY want to do that?",
		"Really Quit??",		
		"&Yes", "&No", QString::null, 1, 1 )  )
	{  
	case 0:
	    emit gotQuit();
	    break;
	default:
	    break;
	}
    }
    else
      emit gotQuit();
}

void
pocketsphinxAdapterMainWindow::gotToggle(int state)
{
    if (state == Qt::Checked)
    {
        emit overwriteEnable();        
    }
    else
    {
        emit overwriteDisable();
    }
}

void
pocketsphinxAdapterMainWindow::setPublishEnable()
{    
    publishStatusCheckBox->setChecked(true);
}

void
pocketsphinxAdapterMainWindow::setPublishDisable()
{
    publishStatusCheckBox->setChecked(false);
}

int
main(int argc, char** argv) {
    QApplication myapp(argc, argv);
    boost::shared_ptr<PocketSphinxGlobalConfig> myConfig;
    if (argc > 1) {
        parseConfigFile myConfigFileParser(argv[1]); //I know, argv[1] may be NULL     
        myConfigFileParser.doParse();
        myConfig = myConfigFileParser.getConfig();
    } else {
        parseConfigFile myConfigFileParser("");
        myConfig = myConfigFileParser.getConfig(); //This essentially means: get defaults
    }    
    pocketsphinxAdapterMainWindow w(myConfig);
    w.setGeometry(200, 100, 1024, 800);
    myapp.setActiveWindow(&w);
    w.show();
    QObject::connect(&w, SIGNAL(gotQuit()), &myapp, SLOT(quit()));
    if (myConfig->IsOpenDeviceDirectly())
    {
        w.handleAlsa(Qt::Checked);
    }
    return myapp.exec();
}
