#include "pocketsphinxAdapterConfig.h"


#include <QApplication>
#include <QLabel>
#include <QProgressBar>
#include <QLayout>
#include <QPushButton>
#include <QMainWindow>
#include <QTextEdit>
#include <QSpinBox>
#include <QMessageBox>
#include <QCheckBox>
#include <QLineEdit>
#include <QTabWidget>
#include <boost/shared_ptr.hpp>
#include <boost/make_shared.hpp>
#include "EqualizerColorWidget.h"
#include <pocketsphinx/pocketsphinx.h>
#include "PocketSphinxWrapper.h"
#include "AlsaMicGrabber.h"
#include "PocketSphinxWidget.h"
#include "SimpleDoubleThresholdSegmenter.h"
#include "SimpleTextAddField.h"
#include "rsbRemoteReceiver.h"
#include "configFileParser.h"

class pocketsphinxAdapterMainWindow : public QMainWindow
{
	Q_OBJECT

private:
	float maxDb;
	float averageDb;
	float stAverageDb;
        bool connectedToRemote;
        unsigned int psInstanceCount;
	boost::shared_ptr<QFrame> baseFrame;
	boost::shared_ptr<QVBoxLayout> vBoxMainLayout;
        boost::shared_ptr<simpleDoubleThresholdSegmenter> simpleSegmenterWidget;
        boost::shared_ptr<QHBoxLayout> hBoxAlsaControlLayout;
	boost::shared_ptr<EqualizerColorWidget> equalizerWidget;
	boost::shared_ptr<QLabel> dbLabel;
	boost::shared_ptr<QLabel> wellLabel;
	boost::shared_ptr<QLabel> clippedCountLabel;
        boost::shared_ptr<QLabel> alsaDeviceLabel;
        boost::shared_ptr<QLineEdit> alsaDeviceLineEdit;
        boost::shared_ptr<QCheckBox> openAlsaDeviceCheckBox;
        boost::shared_ptr<QCheckBox> publishStatusCheckBox;
        boost::shared_ptr<QTabWidget> psResultsTab;
	std::vector<boost::shared_ptr <simpleTextAddField> > psResultsTextEdit;
        boost::shared_ptr<QVBoxLayout> pWidgetLayout;
        boost::shared_ptr<QPushButton> addPWidget;
	boost::shared_ptr<QPushButton> quitButton;
        std::vector<boost::shared_ptr<pocketSphinxWidget> > pWidget;
        boost::shared_ptr<rsbRemoteReceiver> myRemoteReceiver;
        boost::shared_ptr<AlsaMicGrabber> amg;        

/*protected:
    virtual void paintEvent(QPaintEvent *event);*/

signals:
	void gotQuit();
        void overwriteEnable();
        void overwriteDisable();

public slots:
	void update_eq_string (float val, int db_start, int db_utt, QString phase);
        void update_eq_only (float val, bool clipped);
	void quit ();
        void handleAlsa (int state);
        void addPocketSphinxWidget();
        void psAddToText(int id);
        void setPublishEnable();
        void setPublishDisable();
        void gotToggle(int state);
public:
    pocketsphinxAdapterMainWindow(QWidget *parent = 0);
    pocketsphinxAdapterMainWindow(boost::shared_ptr<PocketSphinxGlobalConfig> myConfig,QWidget *parent = 0);
    ~pocketsphinxAdapterMainWindow();
};