/* 
 * File:   simpleDoubleThresholdSegmenter.cpp
 * Author: mschoepf
 * 
 * Created on May 15, 2013, 4:27 PM
 */

#include <qt4/QtGui/qboxlayout.h>
#include <qt4/Qt/qspinbox.h>
#include <qt4/Qt/qpushbutton.h>

#include "SimpleDoubleThresholdSegmenter.h"

void
simpleDoubleThresholdSegmenter::setupUI() {    
    mainLayout = boost::make_shared<QHBoxLayout>();
    groupBox = boost::make_shared<QGroupBox>(QString("Threshold Segmenter Settings"),this);
    groupBoxLayout = boost::make_shared<QHBoxLayout>();
    startThresholdLabel = boost::make_shared<QLabel>("Start (dB): ");
    uttThresholdLabel = boost::make_shared<QLabel> ("Utt (dB): ");
    pauseLabel = boost::make_shared<QLabel> ("Silence (ms): ");
    timeoutLabel = boost::make_shared<QLabel> ("Timeout (s):");
    startThresholdSpinBox = boost::make_shared<QSpinBox> ();
    startThresholdSpinBox->setRange(SIMPLETHRESHOLD_STARTDB_MIN,SIMPLETHRESHOLD_STARTDB_MAX);
    startThresholdSpinBox->setValue(static_cast<int>(startThreshold));
    startThresholdSpinBox->setSingleStep(1);
    uttThresholdSpinBox = boost::make_shared<QSpinBox>();
    uttThresholdSpinBox->setRange(SIMPLETHRESHOLD_UTTDB_MIN,SIMPLETHRESHOLD_UTTDB_MAX);
    uttThresholdSpinBox->setValue(static_cast<int>(uttThreshold));
    uttThresholdSpinBox->setSingleStep(1);
    pauseSpinBox = boost::make_shared<QSpinBox>();
    pauseSpinBox->setRange(SIMPLETHRESHOLD_PAUSE_MIN,SIMPLETHRESHOLD_PAUSE_MAX);
    pauseSpinBox->setValue(static_cast<int>(maxPause));
    pauseSpinBox->setSingleStep(50);
    timeoutSpinBox = boost::make_shared<QSpinBox>();
    timeoutSpinBox->setRange(SIMPLETHRESHOLD_MAXLENGTH_MIN,SIMPLETHRESHOLD_MAXLENGTH_MAX);
    timeoutSpinBox->setValue(static_cast<int>(timeout));
    timeoutSpinBox->setSingleStep(1);
    activateSettingsButton = boost::make_shared<QPushButton>("Set");        
    connect (startThresholdSpinBox.get(), SIGNAL ( valueChanged(int) ), this, SLOT (checkUttOnStart(int) ));
    connect (uttThresholdSpinBox.get(), SIGNAL ( valueChanged (int) ), this, SLOT (checkStartOnUtt(int) ));
    connect (activateSettingsButton.get(), SIGNAL ( clicked() ), this, SLOT (setVariables() ));
    groupBoxLayout->addWidget(startThresholdLabel.get());
    groupBoxLayout->addWidget(startThresholdSpinBox.get());
    groupBoxLayout->addWidget(uttThresholdLabel.get());
    groupBoxLayout->addWidget(uttThresholdSpinBox.get());
    groupBoxLayout->addWidget(pauseLabel.get());
    groupBoxLayout->addWidget(pauseSpinBox.get());
    groupBoxLayout->addWidget(timeoutLabel.get());
    groupBoxLayout->addWidget(timeoutSpinBox.get());
    groupBoxLayout->addWidget(activateSettingsButton.get());
    groupBox->setLayout(groupBoxLayout.get());
    mainLayout->addWidget(groupBox.get());
    this->setLayout(mainLayout.get());    
}

simpleDoubleThresholdSegmenter::simpleDoubleThresholdSegmenter() : maxPause(300),timeout(10), level (0), lengthThisSample (0){
    setupUI();
}

simpleDoubleThresholdSegmenter::simpleDoubleThresholdSegmenter(unsigned int startDb, unsigned int uttDb, unsigned int maxPause, unsigned int timeout) : maxPause(maxPause), timeout(timeout),level (0), lengthThisSample (0){
    startThreshold = static_cast<float>(startDb);
    uttThreshold = static_cast<float>(uttDb);
    std::cout << "maxPause = " << maxPause;
    std::cout << "timeout = " << timeout;    
    setupUI();
    timeoutInSamples = static_cast<unsigned int>(((sampleRate*timeout))/sampleSize);
    maxPauseInSamples = static_cast<unsigned int>(((sampleRate*maxPause)/1000)/sampleSize);
}


simpleDoubleThresholdSegmenter::~simpleDoubleThresholdSegmenter() {
    disconnect (startThresholdSpinBox.get(), SIGNAL (valueChanged(int) ), this, SLOT (checkUttOnStart(int) ));
    disconnect (uttThresholdSpinBox.get(), SIGNAL (valueChanged (int) ), this, SLOT (checkStartOnUtt(int) ));
    disconnect (activateSettingsButton.get(), SIGNAL ( clicked() ), this, SLOT (setVariables() ));
}

void
simpleDoubleThresholdSegmenter::checkStartOnUtt(int i)
{
    if (i < startThresholdSpinBox->value())
        startThresholdSpinBox->setValue(i);
}

void
simpleDoubleThresholdSegmenter::checkUttOnStart(int i)
{
    if (i > uttThresholdSpinBox->value())
        uttThresholdSpinBox->setValue(i);
}

void
simpleDoubleThresholdSegmenter::setVariables()
{
    startThreshold = static_cast<float>(startThresholdSpinBox->value());
    uttThreshold = static_cast<float> (uttThresholdSpinBox->value());
    maxPauseInSamples = static_cast<unsigned int>(((sampleRate*pauseSpinBox->value())/1000)/sampleSize);
    timeoutInSamples = static_cast<unsigned int>(((sampleRate*timeoutSpinBox->value()))/sampleSize);
    timeout = timeoutSpinBox->value();
    maxPause = pauseSpinBox->value();
    emit uttDiscard();
}
void
simpleDoubleThresholdSegmenter::getDBandClipped(float db, bool clipped) 
{    
    switch (level) {
        case 0: //We are in a silence/background state                
            if (db > startThreshold) //We then need to start a segment
            {
                level = 1;
                emit uttStart();
                lengthThisSample = 1;
                lengthBelowUtt = 0;
            }
            break;

        case 1: //We have seen db > startThreshold but not db > uttThreshold yet
            if (db > uttThreshold) //yeah, we have an utterance!
            {
                level = 2;
                lengthThisSample++;
                lengthBelowUtt = 0;
            }
            else
            {
                //We need to decide if we discard the samples
                if ((++lengthThisSample > maxPauseInSamples) || (startThreshold > db) || (lengthThisSample > timeoutInSamples))
                {
                    emit uttDiscard();
                    level = 0;
                }                        
            }
            break;
        case 2: //We have seen db > uttThreshold
            if (lengthThisSample > timeoutInSamples)
            {
                emit uttEnd();
                level = 0;
            }
            if (db > uttThreshold)
            {
                lengthThisSample++;
                lengthBelowUtt=0;
            }
            else
            {
                if (++lengthBelowUtt > maxPauseInSamples)//We have seen enough silence
                {                    
                    emit uttEnd();
                    level = 0;                    
                }
            }
    }
}