/* 
 * File:   configFileParser.h
 * Author: mschoepf
 *
 * Created on June 19, 2013, 3:47 PM
 */

#ifndef CONFIGFILEPARSER_H
#define	CONFIGFILEPARSER_H

#include <iostream>
#include <map>
#include <string>
#include <utility>
#include <fstream>
#include <string>
#include <boost/spirit/include/qi.hpp>
#include <boost/spirit/include/phoenix.hpp>
#include <boost/spirit/include/phoenix_container.hpp>
#include <boost/fusion/include/std_pair.hpp>
#include <boost/fusion/include/adapt_struct.hpp>
#include <boost/fusion/include/vector.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>

#include "defaults.h"

namespace qi = boost::spirit::qi;
namespace px = boost::phoenix;


typedef std::pair<std::string, std::string> pair_type;
typedef std::vector<pair_type> pairs_type;
typedef std::pair<pair_type,pairs_type> sectionT;

 
template<typename Iterator>
struct comment_skipper : public qi::grammar<Iterator> {

    comment_skipper() : comment_skipper::base_type(skip) {
        skip = qi::ascii::space | (qi::lit('#') >> *(qi::char_ - "\n"));
    }
    qi::rule<Iterator> skip;
};


template <typename Iterator,typename Skipper = comment_skipper<Iterator> >
struct key_value_sequence
  : qi::grammar<Iterator, std::vector<sectionT>(), Skipper>
{
    key_value_sequence()
      : key_value_sequence::base_type(sections)
    {
        sections %= *(section);
        section = (sectionHeader >> query);
        sectionHeader = qi::lit('[') >> pair >> qi::lit(']');
        query =  pair >> *((qi::lit(';') | '&') >> pair) >> qi::lit(';');
        pair  =  (key >> -('=' >> value));
        key   =  qi::char_("a-zA-Z_") >> *qi::char_("a-zA-Z_0-9.");
        value = (*qi::lit(' ') >>qi::lexeme[qi::lit('"') >> *qi::char_("/a-zA-Z_0-9. ") >> qi::lit('"')] | +qi::char_("/a-zA-Z_0-9.") );
        
        sections.name("sections");
        section.name("section");
        sectionHeader.name("sectionHeader");
        query.name("query");
        pair.name("pair");
        key.name("key");
        value.name("value");
        qi::debug(sections);
        qi::debug(section);
        qi::debug(sectionHeader);
        qi::debug(query);
        qi::debug(pair);
        qi::debug(key);
        qi::debug(value);
        
    }
    qi::rule<Iterator, std::vector<sectionT>(), Skipper> sections;
    qi::rule<Iterator, sectionT(), Skipper> section;
    qi::rule<Iterator, pair_type(),Skipper> sectionHeader;
    qi::rule<Iterator, pairs_type(),Skipper> query;
    qi::rule<Iterator, pair_type(),Skipper> pair;
    qi::rule<Iterator, std::string(),Skipper> key, value;
    
};

#include <iostream>
#include <exception>

class PocketSphinxParseException: public std::runtime_error {
    public:    
    PocketSphinxParseException(std::string const &s):std::runtime_error(s) {}
};



class PocketSphinxInstanceConfig {
public:
    
    PocketSphinxInstanceConfig();
    void parseSection(sectionT mySection);
    
    bool IsAutoload() const {
        return autoload;
    }

    void SetAutoload(bool autoload) {
        this->autoload = autoload;
    }

    bool IsBestpath() const {
        return bestpath;
    }

    void SetBestpath(bool bestpath) {
        this->bestpath = bestpath;
    }

    std::string GetDictDir() const {
        return dictDir;
    }

    void SetDictDir(std::string dictDir) {
        this->dictDir = dictDir;
    }

    std::string GetHmmDir() const {
        return hmmDir;
    }

    void SetHmmDir(std::string hmmDir) {
        this->hmmDir = hmmDir;
    }

    std::string GetJsgfDir() const {
        return jsgfDir;
    }

    void SetJsgfDir(std::string jsgfDir) {
        this->jsgfDir = jsgfDir;
    }

    std::string GetLmDir() const {
        return lmDir;
    }

    void SetLmDir(std::string lmDir) {
        this->lmDir = lmDir;
    }

    std::string GetName() const {
        return name;
    }

    void SetName(std::string name) {
        this->name = name;
    }

    std::string GetRsbScope() const {
        return rsbScope;
    }

    void SetRsbScope(std::string rsbScope) {
        this->rsbScope = rsbScope;
    }

private:
    std::string name;
    std::string hmmDir;
    std::string lmDir;
    std::string dictDir;
    std::string jsgfDir;
    std::string rsbScope;
    bool bestpath;
    bool autoload;
};

class PocketSphinxGlobalConfig {
public:
    PocketSphinxGlobalConfig();
    
    void parseSection (sectionT globalSection);        
            
    
    std::string GetAlsaDevice() const {
        return alsaDevice;
    }

    void SetAlsaDevice(std::string alsaDevice) {
        this->alsaDevice = alsaDevice;
    }

    bool IsOpenDeviceDirectly() const {
        return openDeviceDirectly;
    }

    void SetOpenDeviceDirectly(bool openDeviceDirectly) {
        this->openDeviceDirectly = openDeviceDirectly;
    }

    std::vector<boost::shared_ptr<PocketSphinxInstanceConfig> > GetPocketSphinxInstanceVector() const {
        return pocketSphinxInstanceVector;
    }

    void AddToPocketSphinxInstanceVector(boost::shared_ptr<PocketSphinxInstanceConfig> myPocketSphinxInstanceConfig) {
        this->pocketSphinxInstanceVector.push_back(myPocketSphinxInstanceConfig);
    }

    unsigned int GetSilence() const {
        return silence;
    }

    void SetSilence(unsigned int silence) {
        this->silence = silence;
    }

    unsigned int GetStartDB() const {
        return startDB;
    }

    void SetStartDB(unsigned int startDB) {
        this->startDB = startDB;
    }

    unsigned int GetTimeout() const {
        return timeout;
    }

    void SetTimeout(unsigned int timeout) {
        this->timeout = timeout;
    }

    unsigned int GetUttDB() const {
        return uttDB;
    }

    void SetUttDB(unsigned int uttDB) {
        this->uttDB = uttDB;
    }

private:
    unsigned int startDB;
    unsigned int uttDB;
    unsigned int silence;
    unsigned int timeout;
    std::string alsaDevice;
    bool openDeviceDirectly;
    std::vector<boost::shared_ptr<PocketSphinxInstanceConfig> > pocketSphinxInstanceVector;
};


class parseConfigFile {
public:
    parseConfigFile (const std::string &filename);
    void doParse();
    boost::shared_ptr<PocketSphinxGlobalConfig> getConfig();
private:
    std::string filename;
    boost::shared_ptr<PocketSphinxGlobalConfig> myConfig;    
};
#endif	/* CONFIGFILEPARSER_H */

