/* 
 * File:   listenToRsb.cpp
 * Author: mschoepf
 *
 * Created on June 17, 2013, 7:56 PM
 */

#include <cstdlib>
#include <boost/shared_ptr.hpp>
#include <boost/make_shared.hpp>
#include <rsb/Factory.h>
#include <rsc/rscexports.h>
#include <rsb/converter/Repository.h>
#include <rsb/converter/ProtocolBufferConverter.h>
#include <rsbxml0.9/rsbxml/XOPConverter.h>
#include <rsbxml0.9/rsbxml/XOP.h>

extern "C" {
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
}

using namespace std;

xmlXPathObjectPtr
getnodeset (xmlDocPtr doc, xmlChar *xpath){
	
	xmlXPathContextPtr context;
	xmlXPathObjectPtr result;

	context = xmlXPathNewContext(doc);
	if (context == NULL) {
		printf("Error in xmlXPathNewContext\n");
		return NULL;
	}
	result = xmlXPathEvalExpression(xpath, context);
	xmlXPathFreeContext(context);
	if (result == NULL) {
		printf("Error in xmlXPathEvalExpression\n");
		return NULL;
	}
	if(xmlXPathNodeSetIsEmpty(result->nodesetval)){
		xmlXPathFreeObject(result);
                printf("No result\n");
		return NULL;
	}
	return result;
}

/*
 * 
 */
int main(int argc, char** argv) {
    
    LIBXML_TEST_VERSION
    
    xmlDocPtr doc;
    
    rsb::Scope scope(argc > 1 ? argv[1] : "/speechrec/psa");
    std::cout << "Listening on scope " << scope << std::endl;
    boost::shared_ptr<rsb::converter::XOPConverter> myXopConverter = boost::make_shared<rsb::converter::XOPConverter>();    
    rsb::converter::converterRepository<std::string>()->registerConverter(myXopConverter);
    
    // Create a reader which synchronously receives events on the
    // specified scope.
    rsb::Factory& factory = rsb::getFactory();
    rsb::ReaderPtr reader = factory.createReader(scope);

    // Print events as they are received.
    while (true) {
        rsb::EventPtr event = reader->read();
        //std::cout << event << std::endl;
        boost::shared_ptr<rsb::transport::XOP> myXOP = boost::static_pointer_cast<rsb::transport::XOP> (event->getData());
        //std::cout << myXOP->getDocumentText() << std::endl;

        const char* buffer = new char[myXOP->getDocumentText().size()];
        doc = xmlReadMemory(myXOP->getDocumentText().data(), myXOP->getDocumentText().size(), "noname.xml", NULL, 0);
        if (doc == NULL) {
            std::cerr << "Failed to parse document: " << myXOP->getDocumentText() << std::endl;
        }
        else {
            xmlChar *xpath = (xmlChar*) "//part";
            xmlNodeSetPtr nodeset;
            xmlXPathObjectPtr result;
            int i;
            xmlChar *keyword;            
            result = getnodeset(doc, xpath);
            if (result) {
                nodeset = result->nodesetval;
                for (i = 0; i < nodeset->nodeNr; i++) {
                    keyword = xmlNodeListGetString(doc, nodeset->nodeTab[i]->xmlChildrenNode, 1);
                    std::cout <<  keyword << " ";
                    xmlFree(keyword);
                }
                std::cout << std::endl;
                xmlXPathFreeObject(result);
            }
            xmlFreeDoc(doc);
            

        }
    }
    xmlCleanupParser();
}


