/* 
 * File:   rsbRemoteReceiver.cpp
 * Author: mschoepf
 * 
 * Created on June 17, 2013, 2:54 PM
 */

#include <boost/smart_ptr/make_shared.hpp>

#include "rsbRemoteReceiver.h"

rsbCallbackReceiver::rsbCallbackReceiver() : toggle (false), state (1){
}

boost::shared_ptr<std::string>
rsbCallbackReceiver::call(const std::string& methodname, boost::shared_ptr<std::string> input) {
    boost::shared_ptr<std::string> reply = boost::make_shared<std::string>("Ok");
    std::cout << "Called method " << methodname << " with argument " << input->c_str() << std::endl;
    {
        boost::lock_guard<boost::mutex> lock (myMutex);
        int newState = -1;
        if (0 == input->compare("0"))
        {
            newState = 0;
        }
        if (0 == input->compare("1"))
        {
            newState = 1;
        }
        if (-1 == newState)
        {
            std::cerr << "Got argument: '" << input << "' which is unknown" << std::endl;
            reply->assign("Unkown command");
            return reply;
        }
        if (state != newState) {
            std::cout << "Toggled Input" << std::endl;
            toggle = true;
            state = newState;
        }
    }
    std::cout << "Notifying" << std::endl;
    condWaitForRemote.notify_all();
    return reply;
}

void
rsbCallbackReceiver::resetToggle() {
    toggle = false;
}

bool 
rsbCallbackReceiver::hasToggled() {
    return toggle;
}

int
rsbCallbackReceiver::getState() {
    return state;
}

void
rsbCallbackReceiver::setState(int newstate) {
    state = newstate;
}


rsbRemoteReceiver::rsbRemoteReceiver(const std::string &scope) : scope(scope) {    
    rsb::Factory & factory = rsb::getFactory();
    std::cout << "Server listening on " << scope << std::endl;
    server = factory.createServer(scope);
    myReceiver = boost::make_shared<rsbCallbackReceiver>();
    server->registerMethod("speechRecEnable",myReceiver );
    remoteListenThread = boost::make_shared<boost::thread>(&rsbRemoteReceiver::runner,this);
}

rsbRemoteReceiver::~rsbRemoteReceiver() {
}

void
rsbRemoteReceiver::runner(){
    while (true)
    {
        std::cout << "Trying to lock " << std::endl;
        boost::unique_lock<boost::mutex> lock (myReceiver->myMutex);
        while (!myReceiver->hasToggled()){
            std::cout << "before wait" << std::endl;
            myReceiver->condWaitForRemote.wait(lock);
            std::cout << "after wait" << std::endl;
        }
        if (myReceiver->hasToggled())
        {
            int state = myReceiver->getState();
            if (1==state)
            {
                std::cout << "Enabling output" << std::endl;
                emit remoteEnable();
            }
            else
            {
                std::cout << "Disabling output" << std::endl;
                emit remoteDisable();
            }
            myReceiver->resetToggle();
        }        
    }
}

void
rsbRemoteReceiver::overwriteDisable()
{
    myReceiver->setState(0);
    emit remoteDisable();
}

void
rsbRemoteReceiver::overwriteEnable()
{
    myReceiver->setState(1);
    emit remoteEnable();
}